% This script replicates Figure 3 in EZC
%
clear
clc
% Calibration parameters
eta = -6:.1:6;                    % Deterministic Sample
c1 = mean(eta); c2 = mean(eta)+1; % Two location parameters
s = -3:0.05:3;                    % An exogenous trasition variable 
eta_t = (s - c1).*(s-c2);         % The new nonlinear kernel
%eta_t=eta;

%Case 1: gamma1 varies, gamma2 constant to 0
[ MU1, G1] = h_gamma(eta_t, -4, 0);
[ MU2, G2] = h_gamma(eta_t, -2, 0);
[ MU3, G3] = h_gamma(eta_t, 0, 0);
[ MU4, G4] = h_gamma(eta_t, 2, 0);
[ MU5, G5] = h_gamma(eta_t, 4, 0);

%Case 2: gamma1 constant to 0, gamma2 varies;
[ MU6, G6] = h_gamma(eta_t, 0, -4);
[ MU7, G7] = h_gamma(eta_t, 0, -2);
[ MU8, G8] = h_gamma(eta_t, 0, 0);
[ MU9, G9] = h_gamma(eta_t, 0, 3);
[ MU10, G10] = h_gamma(eta_t, 0, 6);

%PLOT RESULTS

figure(1)
subplot(2,1,1)
p1 = plot(eta, G1, eta, G2, eta, G3, eta, G4, eta, G5);
xlabel('\eta_t')                          
ylabel('G(\gamma, h(\eta_t))')                       
title('GLSTAR2: changes in \gamma1 ')
legend('\gamma1= -6', '\gamma1= -3', '\gamma1= 0', '\gamma1= 3','\gamma1= 6', 0)
set(p1,'LineWidth',2)
subplot(2,1,2)
axis tight;
subplot(2,1,2)
p2 = plot(eta, G6, eta, G7, eta, G8, eta, G9, eta, G10);
xlabel('\eta_t')                         
ylabel('G(\gamma, h(\eta_t))')                       
title('GLSTAR2: changes in \gamma2 ')  
legend('\gamma2= 6', '\gamma2= -3', '\gamma2= 0', '\gamma2= 3', '\gamma2= 6', 0)
set(p2,'LineWidth',2)
axis tight;