% This file perform the simulation for test for SYMMETRY in a GSTAR model
%
clear
clc
randn('state', 535621);
%% STEP 0: Define the parameters
%Linear Part
% ORIGINAL    
phi0 = 0;   
phi1 = .8;  
phi2 = -.7; 
%Nonlinear Linear Part
theta0 = 0.01; 
theta1 = -.9; 
theta2 = .795;  
%Slopes for different tails
gamma1 = 50;
gamma2 = 10;
%Other parameter for simulation
T = 100;
nsim = 1000;
alpha01 = 0.01; alpha05 = 0.05; alpha10 = 0.1;
sigerr = 1.0; 
d= 100; %Discard the first d simulations

%Preallocate
Y=zeros(T-2,nsim);
P1=zeros(1, nsim); P2=zeros(1, nsim);

SS1=zeros(1,nsim); SS2=zeros(1, nsim);
Beta = zeros(1, nsim);
Power = zeros(1, nsim);

%% STEP1: INITIALIZATION
%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0, phi1, phi2, T , sigerr);

%Generate G
[theta, eta, s, c, eta_t, h, mu, G] = sim_G_glstar1(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);
%plot(sort(s),sort(G));

%Build up the elements of the Score and Hessian (for whom I need only the
%parameters gamma1 gamma2, T, eta and eta_t which in turns are function
%of the Linear AR model previously defined.
[Ggamma1] = Ggamma1(gamma1, gamma2, h,  eta, eta_t, T);
[Ggamma2] = Ggamma2(gamma1, gamma2, h,  eta, eta_t, T);
[Gc] = Gc(gamma1, gamma2, h, eta, eta_t,  T);

%% STEP 2: SIMULATION
for S=1:nsim 
   
%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0, phi1, phi2, T , sigerr);
u_l=u_l(3:end,1);
 
%Generate G (select "sim_G_glstar1" for Generalized Logistic and
% "sim_G_gestar" for Generalized Exponential.
%[theta, eta, s, c, eta_t, h, mu, G]=sim_G_glstar1(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);
[theta, eta, s, c, eta_t, h, mu, G]=sim_G_gestar(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);
%Generate the NON-LINEAR part of the process in order to have w which is required for G and the
%Hessian
[y_nl, u_nl] = sim_nlAR(theta0, theta, G, T, sigerr);

%Define the GLSTAR1 as the sum of the Linear AR + Nonlinear AR previously
%defined
[y_gstar, u, sigma2_hat, sigma_hat] = nsim_GLSTAR1(T, y, y_nl, u_nl);
Y(:,S)=y_gstar;

%Define the two symmetry tests (select "nLinearitySLT_test" for LM1 and 
% "nLinearitySLT_ESTARtest" for LM2 statistics)
[S1,  pval_S1] = nSymmetry_test(gamma1, gamma2, w, theta0,theta1,theta2, Ggamma1, Ggamma2, u);
%[hat_z, h0, z_nl, RES0, SSR0, sigma20, RES1, SSR1, sigma21, S2, CDF, pval_S2] = nLinearitySLT_test(y_gstar, phi0, phi, theta0, theta, gamma1, gamma2, T, s, c);
[hat_z, h0, z_nl, RES0, SSR0, sigma20, RES1, SSR1, sigma21,S2, CDF, pval_S2] = nLinearitySLT_ESTARtest(y_gstar, phi0, phi, theta0, theta, gamma1, gamma2, T, s, c);
%And store the two statistics
SS1(1,S) = S1; 
SS2(1,S) = S2; 
P1(1,S) = pval_S1; 
P2(1,S) = pval_S2;
end
%Discard the first d simulations
Y(:,1:d)=[]; 
P1(:,1:d)=[]; 
P2(:,1:d)=[]; 
%Compute Empirical P-values
EPval_S1= mean(P1); EPval_S2= mean(P2);
%Define the Type II error (Beta)
NS=1:size(Y,2); N=length(NS);
Beta_S1_01=cumsum(P1>alpha01)./NS; Beta_S2_01=cumsum(P2>alpha01)./NS;
Beta_S1_05=cumsum(P1>alpha05)./NS; Beta_S2_05=cumsum(P2>alpha05)./NS;
Beta_S1_10=cumsum(P1>alpha10)./NS; Beta_S2_10=cumsum(P2>alpha10)./NS;
%        Powers
Power_S1_01= 1-Beta_S1_01; Power_S2_01= 1-Beta_S2_01;
Power_S1_05= 1-Beta_S1_05; Power_S2_05= 1-Beta_S2_05;
Power_S1_10= 1-Beta_S1_10; Power_S2_10= 1-Beta_S2_10;
%        Empirical Betas
EBeta_S1_01=mean(Beta_S1_01); EBeta_S2_01=mean(Beta_S2_01);
EBeta_S1_05=mean(Beta_S1_05); EBeta_S2_05=mean(Beta_S2_05);
EBeta_S1_10=mean(Beta_S1_10); EBeta_S2_10=mean(Beta_S2_10);
%        Empirical Power
EP_S1_01=mean(Power_S1_01); EP_S2_01=mean(Power_S2_01);
EP_S1_05=mean(Power_S1_05); EP_S2_05=mean(Power_S2_05);
EP_S1_10=mean(Power_S1_10); EP_S2_10=mean(Power_S2_10);
%        Empirical Standard Error
ESE_S1_01= mean(((Power_S1_01 - EP_S1_01).^2)/sqrt(T)); ESE_S2_01= mean(((Power_S2_01 - EP_S2_01).^2))/sqrt(T);
ESE_S1_05= mean(((Power_S1_05 - EP_S1_05).^2)/sqrt(T)); ESE_S2_05= mean(((Power_S2_05 - EP_S2_05).^2))/sqrt(T);
ESE_S1_10= mean(((Power_S1_10 - EP_S1_10).^2)/sqrt(T)); ESE_S2_10= mean(((Power_S2_10 - EP_S2_10).^2))/sqrt(T);
%%%% RESULTS %%%%%%%%
disp('*******************************************************************')
disp('                         Monte Carlo Results                       ')
disp('                              SYMMETRY                             ')
disp('*******************************************************************')
disp('-------------------------------------------------------------------')
disp('                           alpha = 0.01                      ')
disp('-------------------------------------------------------------------')
disp('                                                             ')
disp('           TWO-STEP TEST                 |       ALL-IN-ONE TEST      ')
disp('---------------------------------------  | -----------------------------------')
disp(' Empirical  |  Emp.   |   Emp.  | Emp.   |   Emp.  |   Emp.  |  Emp.   |  Emp.')
disp('  p-value   |  Beta   |  Power  | S.E.   | p-value |  Beta   | Power   |  S.E.')
disp('    ------  | ------  | ------  |------- | ------  | ------  | ------  | ------')
disp( [EPval_S1 EBeta_S1_01 EP_S1_01 ESE_S1_01 EPval_S2 EBeta_S2_01 EP_S2_01 ESE_S2_01])
disp('-------------------------------------------------------------------')
disp('                           alpha = 0.05                            ')
disp('-------------------------------------------------------------------')
disp('                          ')
disp('           TWO-STEP TEST                 |       ALL-IN-ONE TEST      ')
disp('---------------------------------------  | -----------------------------------')
disp(' Empirical  |  Emp.   |   Emp.  | Emp.   |   Emp.  |   Emp.  |  Emp.   |  Emp.')
disp('  p-value   |  Beta   |  Power  | S.E.   | p-value |  Beta   | Power   |  S.E.')
disp('    ------  | ------  | ------  |------- | ------  | ------  | ------  | ------')
disp( [EPval_S1 EBeta_S1_05 EP_S1_05 ESE_S1_05 EPval_S2 EBeta_S2_05 EP_S2_05 ESE_S2_05])
disp('-------------------------------------------------------------------')
disp('                           alpha = 0.10                            ')
disp('-------------------------------------------------------------------')
disp('                          ')
disp('           TWO-STEP TEST                 |       ALL-IN-ONE TEST      ')
disp('---------------------------------------  | -----------------------------------')
disp(' Empirical  |  Emp.   |   Emp.  | Emp.   |   Emp.  |   Emp.  |  Emp.   |  Emp.')
disp('  p-value   |  Beta   |  Power  | S.E.   | p-value |  Beta   | Power   |  S.E.')
disp('    ------  | ------  | ------  |------- | ------  | ------  | ------  | ------')
disp( [EPval_S1 EBeta_S1_10 EP_S1_10 ESE_S1_10 EPval_S2 EBeta_S2_10 EP_S2_10 ESE_S2_10])
disp('-------------------------------------------------------------------')


%figure(1)
%plot(NS, Beta_S1_01, NS, Power_S1_01,'-o', NS, Beta_S2_01, NS, Power_S2_01,'-o');
%legend('Beta S1 (\alpha=0.01)', 'Power S1 (\alpha=0.01)', 'Beta S2 (\alpha=0.01)', 'Power S2 (\alpha=0.01)');