% This file perform the simulation for accuracy of the estimation in GSTAR model
% corresponding to Table 1 in Supplement of Zanetti Chini (2018).
%
clear
clc
randn('state', 535621);
%% STEP 0: Define the parameters
%Linear Part
% ORIGINAL    
phi0 = 0;   
phi1 = .65;  
phi2 = -.5; 
%Nonlinear Linear Part
theta0 = 0.02; 
theta1 = -.8; 
theta2 = .4;  
%Slopes for different tails
gamma1 = 2;
gamma2 = 2;
%Other parameter for simulation
T = 25;
nsim = 1000;
alpha01 = 0.01; alpha05 = 0.05; alpha10 = 0.1;
sigerr = 1.0; 
d= 100; %Discard the first d simulations

%Preallocate
Y= zeros(T-2,nsim);
Gamma1 = zeros(1,nsim);
Gamma2 = zeros(1,nsim);
C = zeros(1,nsim);
%% STEP1: INITIALIZATION
%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0, phi1, phi2, T , sigerr);
%
%Generate G
[theta, eta, s, c, eta_t, h, mu, G] = sim_G_glstar1(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);
%plot(sort(s),sort(G));
%
%Build up the elements of the Score and Hessian (for whom I need only the
%parameters gamma1 gamma2, T, eta and eta_t which in turns are function
%of the Linear AR model previously defined.
[Ggamma1] = Ggamma1(gamma1, gamma2, h,  eta, eta_t, T);
[Ggamma2] = Ggamma2(gamma1, gamma2, h,  eta, eta_t, T);
[Gc] = Gc(gamma1, gamma2, h, eta, eta_t,  T);
%
%% STEP 2: SIMULATION
for S=1:nsim 
%  
%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0, phi1, phi2, T , sigerr);
% 
%Generate G (select "sim_G_glstar1" for Generalized Logistic and
% "sim_G_gestar" for Generalized Exponential.
%[theta, eta, s, c, eta_t, h, mu, G]=sim_G_glstar1(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);
[theta, eta, s, c, eta_t, h, mu, G]=sim_G_gestar(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);
%Generate the NON-LINEAR part of the process in order to have w which is required for G and the
%Hessian
[y_nl, u_nl] = sim_nlAR(theta0, theta, G, T, sigerr);
%
%Define the GLSTAR1 as the sum of the Linear AR + Nonlinear AR previously
%defined
[y_gstar, u, sigma2_hat, sigma_hat] = nsim_GLSTAR1(T, y, y_nl, u_nl);
%
%% STEP 2: ESTIMATE THE MODEL

O = ones(length(y_gstar),1);
X = [O, lag(y_gstar,1) lag(y_gstar,2)   ]; 
W = [];         
q = lag(y_gstar,1);   
T_l = length(y_gstar);
nX = size(X,2); 
nW = 0;         
M = [];        
rob = 1;        
flag = 1;       
sig = 0.10;      
m=1; 
%initial values
gamma1_0= 1 ;
gamma2_0= 1 ;
c_0 = 0;
% Call the function for parameter estimation
[alpha_gstar,beta_gstar,lambda_gstar,gamma1,gamma2,c_gstar,fX_gstar,yhat_gstar,ehat_gstar,G_gstar,sigma_gstar,se_gstar]=parestGSTARlm(y_gstar,X,W,q,T_l,nX,nW,m,gamma1_0,gamma2_0,c_0);

Y(:,S) = yhat_gstar;
Gamma1(:,S) = gamma1;
Gamma2(:,S) = gamma2;
C(:,S) = c_gstar;
end
%Discard the first d simulations
Y(:,1:d)=[]; 
Gamma1(:,1:d)=[]; 
Gamma2(:,1:d)=[]; 

%Compute Empirical estimated parameters
GAMMA1 = mean(Gamma1); 
GAMMA2 = mean(Gamma2);
C=mean(C);

disp('---------------------------------------------------------------------------')
disp('                          EMPIRICAL PARAMETERs ESTIMATE                       ')
disp('---------------------------------------------------------------------------')
disp('     Emp. |   Emp.   |    Emp.   ')
disp('   gamma_1|  gamma_2 |     c     ')
disp('   -------   ------- |  -------  ')
disp( [GAMMA1  GAMMA2  C])